unit fTestMain;

interface

uses
  Windows, Classes, Controls, Forms, Dialogs, StdCtrls, SysUtils, ActiveX, VCFI;

type

  TForm1 = class;

  TMyThread = class(TThread)
  protected
    FOwner: TForm1;
    FIndex: Integer;
    procedure RefreshTime;
    procedure Execute; override;
      procedure Execute0;
      procedure Execute1;
  public
    constructor Create(AOwner: TForm1; Index: Integer);
  end;

  TForm1 = class(TForm)
    btnStart0: TButton;
    Edit1: TEdit;
    Label1: TLabel;
    btnTerminate0: TButton;
    btnStart1: TButton;
    btnTerminate1: TButton;
    btnTerminate2: TButton;
    procedure btnStart0Click(Sender: TObject);
    procedure btnTerminate0Click(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure btnTerminate1Click(Sender: TObject);
    procedure btnTerminate2Click(Sender: TObject);
  protected
    MyThread: TMyThread;
    procedure DoOnMyThreadTerminated(Sender: TObject);    
  public
  end;

var
  Form1: TForm1;

implementation

{$R *.dfm}

{ TMyThread }

constructor TMyThread.Create(AOwner: TForm1; Index: Integer);
begin
  inherited Create(False);
  FOwner := AOwner;
  FIndex := Index;
  FreeOnTerminate := False;
end;

procedure TMyThread.Execute;
begin
  case FIndex of
    0: Execute0;
    1: Execute1;
  end;
end;

procedure TMyThread.Execute0;
begin
  while not Terminated do
  begin
    Synchronize(RefreshTime);
    Sleep(35);
  end;
  Sleep(100);
end;

procedure TMyThread.Execute1;
var
  VT: TVtChart;
begin
//  CoInitializeEx(nil, COINIT_MULTITHREADED);
  //try
    VT := TVtChart.Create(nil);
    try
      while not Terminated  do
      begin
        Synchronize(RefreshTime);
        Sleep(35);
      end;
      Sleep(100);
    finally
      FreeAndNil(Vt);
    end;
  //finally
    //  CoUninitialize;
  //end;
end;

procedure TMyThread.RefreshTime;
begin
  FOwner.Edit1.Text := DateTimeToStr(now) + IntToStr(Random(991000));
end;

{ TForm1 }

procedure TForm1.DoOnMyThreadTerminated(Sender: TObject);
begin
  if MyThread <> nil then
  begin
    Label1.Caption := 'MyThread Terminated, Index = ' +
                      IntToStr(MyThread.FIndex);
    FreeAndNil(MyThread);
  end;
end;

procedure TForm1.btnTerminate0Click(Sender: TObject);
begin
  if MyThread <> nil then
    MyThread.Terminate;
end;

procedure TForm1.btnStart0Click(Sender: TObject);
begin
  Label1.Caption := '';
  if MyThread = nil then
  begin
    MyThread := TMyThread.Create(Self, TComponent(Sender).Tag);
    MyThread.OnTerminate := DoOnMyThreadTerminated;
  end;
end;

procedure TForm1.FormCreate(Sender: TObject);
begin
  Randomize;
end;

procedure TForm1.btnTerminate1Click(Sender: TObject);
var
  Timeout: Cardinal;
  IsTimeout: Boolean;
begin
  if MyThread <> nil then
  begin
    MyThread.OnTerminate := nil;//
    IsTimeout := False;
    Timeout := GetTickCount + 5000;

    MyThread.Terminate;
    while not (MyThread.Terminated or IsTimeout)do
    begin
      IsTimeout := GetTickCount > timeout;
      Application.ProcessMessages;
      Sleep(10);
    end;

    if IsTimeout then
      Label1.Caption := 'MyThread Termination Timeout'
    else if MyThread.Terminated then
       Label1.Caption := '(NEW)MyThread Terminated, Index = ' +
                          IntToStr(MyThread.FIndex);

    MyThread.Free;
    MyThread := nil;

  end;
end;

procedure TForm1.btnTerminate2Click(Sender: TObject);
begin
  if MyThread <> nil then
  begin

    MyThread.OnTerminate := nil;//

    MyThread.Terminate;
    MyThread.WaitFor;

    if MyThread.Terminated then
       Label1.Caption := '(NEW)MyThread Terminated, Index = ' +
                          IntToStr(MyThread.FIndex);

    MyThread.Free;
    MyThread := nil;

  end;
end;

initialization

  CoInitializeEx(nil, COINIT_MULTITHREADED);

finalization

  CoUninitialize;

end.
